<?php

App::uses('AppComponent', 'Controller/Component');

class ProductsComponent extends AppComponent {

	/**
	 * get a product listing
	 *
	 * @param 	array 	query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @return 	array 	Array of results
	 */
	public function listing($params = array(), $paginate = true) {
		$params = array_merge_recursive(
			array(
				'conditions' => array(
					'Product.is_active' => 1
				),
				'order' => 'CategoriesProduct.sequence ASC, Product.name ASC',
				'contain' => array(
					'Product' => array(
						'ListingImage',
						'Variant' => array(
							'order' => 'Variant.sequence ASC'
						)
					)
				),
				'group' => 'CategoriesProduct.product_id'
			),
			$params
		);

		$CategoriesProduct = EvClassRegistry::init('EvShop.CategoriesProduct');

		if ($paginate && ! empty($this->_controller)) {
			$this->_controller->Paginator->settings = $params;
			return $this->_controller->Paginator->paginate(
				$CategoriesProduct
			);
		} else {
			return $CategoriesProduct->find('all', $params);
		}
	}

	/**
	 * get a featured product listing
	 *
	 * @param 	array 	query / paginate params
	 * @return 	array 	Array of results
	 */
	public function featured($params = array()) {
		$params = array_merge_recursive(
			array(
				'contain' => array(
					'Product' => array(
						'conditions' => array(
							'Product.is_featured' => 1
						)
					)
				)
			),
			$params
		);

		return $this->listing($params, false);
	}

	/**
	 * get a listing by category
	 *
	 * @param 	int|array 	Category ID or array of category IDs
	 * @param 	array 		query / paginate params
	 * @return 	array 		Array of results
	 */
	public function listingByCategory($categoryId, $params = array()) {
		$defaults = array(
			'conditions' => array(
				'CategoriesProduct.category_id' => $categoryId
			)
		);

		// if it's many categories, and the group by and9
		// also contain the categories so we know which is which
		if (is_array($categoryId)) {
			$defaults['contain'][] = 'Category';
		}

		$params = array_merge_recursive(
			$defaults,
			$params
		);

		return $this->listing($params);
	}

	/**
	 * get a listing by a brand
	 *
	 * @param 	int|array 	brand ID or array of brand IDs
	 * @param 	array 		query / paginate params
	 * @return 	array 		Array of results
	 */
	public function listingByBrand($brandId, $params = array()) {
		$defaults = array(
			'contain' => array(
				'Product' => array(
					'conditions' => array(
						'Product.brand_id' => $brandId
					)
				)
			)
		);

		// if it's many categories, and the group by and
		// also contain the categories so we know which is which
		if (is_array($brandId)) {
			$defaults['contain']['Product'][] = 'Brand';
		}

		$params = array_merge_recursive(
			$defaults,
			$params
		);

		return $this->listing($params);
	}
}
